/*
Copyright 2010-2011 Kevin Gimpel

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "dense_vocabulary.h"
#include <iostream>

DenseVocabulary::DenseVocabulary() {
	string nullString = "~~~";
	lookup.set_empty_key(nullString);
}

// Returns boolean indicating whether or not WORD is in the vocabulary.
bool DenseVocabulary::isNew(const WORD& word) {
	dense_hash_map<WORD, WORD_ID, MurmurHash2, eqstring>::const_iterator iter = lookup.find(word);
	return (iter == lookup.end());
}

//WORD_ID DenseVocabulary::
// If WORD is not in the vocabulary, adds it.
// Returns index of WORD in the vocabulary.
// TODO change the following to use iterators and see if this improves performance.
WORD_ID DenseVocabulary::storeIfNew(const WORD& word) {
	WORD_ID id = vocab.size();
	// Try to insert the word into the dense_hash_map.
	pair<dense_hash_map<WORD, WORD_ID, MurmurHash2, eqstring>::iterator, bool> res = lookup.insert(make_pair(word, id));
	if (res.second) {
		// If res.second == true, then word was actually inserted with id as its WORD_ID.
		// Add the word onto the end of the vector.
		vocab.push_back(word);
		return id;
	} else {
		// The word already existed in the hash_map with some WORD_ID.
		// Return its existing WORD_ID.
		return res.first->second;
	}
}

// Adds WORD to the vocabulary.
// Precondition: WORD is not contained in the vocabulary.
// Returns index of WORD in the vocabulary.
WORD_ID DenseVocabulary::storeNew(const WORD& word) {
	WORD_ID id = vocab.size();
	vocab.push_back(word);
	lookup.insert(make_pair(word, id));
	//lookup[word] = id;
	return id;
}

// Returns index of WORD in the vocabulary, or 0 if not found.
WORD_ID DenseVocabulary::getWordID(const WORD& word) {
	dense_hash_map<WORD, WORD_ID, MurmurHash2, eqstring>::const_iterator iter = lookup.find(word);
	if (iter != lookup.end()) {
		return iter->second;
	} else {
		return 0;
	}
}
