/*
Copyright 2010-2011 Kevin Gimpel

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
//
// Class to represent a string vocabulary using Google dense hash maps.
//
// author: Kevin Gimpel
//
// Parts of this class were stolen from a similar class in the Moses MT system.

#ifndef DENSE_VOCABULARY_H_
#define DENSE_VOCABULARY_H_

#include <vector>
#include <cstdlib>
#include <map>
#include "utils.h"
#include "hashing.h"
#include "../external/include/google/dense_hash_map"

using google::dense_hash_map;

typedef string WORD;
typedef unsigned int WORD_ID;

class DenseVocabulary {
	public:
		DenseVocabulary();

		bool isNew(const WORD& word);
		WORD_ID storeIfNew(const WORD& word);
		WORD_ID storeNew(const WORD& word);
		WORD_ID getWordID(const WORD& word);
		inline WORD &getWord( WORD_ID id ) { return vocab[ id ]; }

		dense_hash_map<WORD, WORD_ID, MurmurHash2, eqstring> lookup;  // Maps WORDs to unique IDs.
		vector<WORD> vocab;          // Maps unique IDs to WORDs.
};

#endif /*DENSE_VOCABULARY_H_*/
